<?php
header('Content-Type: text/html');

define('CODE_DIR', '/tmp');

?>
<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0, user-scalable=no">
    <meta http-equiv="X-UA-Compatible" content="ie=edge">
    <title>365FarmNet Connect</title>
    <link rel="shortcut icon" href="favicon.ico" />
    <link rel="stylesheet" href="styles.css">
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.4.1/jquery.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/ace/1.4.7/ace.js" type="text/javascript" charset="utf-8"></script>
</head>

<body>
    <?php
    require __DIR__ . '/vendor/autoload.php';

    use \Firebase\JWT\JWT;

    // your partner credentials with stage-specific secrets
    $partners = array(
        '5726c2cf-143b-4834-aa54-24a1c1516a48' => array(
            'development' => 'trsL26xTtFXgPHBJE8n4ZrN6R7fWfLrK',
            'production' => 'trsL26xTtFXgPHBJE8n4ZrN6R7fWfLrK'
        ) // developer
    );

    // the Connect-Token is added to your application's URL within the platform
    if (isset($_REQUEST['jwt'])) {

        $connect_token = $_REQUEST['jwt'];

        // the Connect-Token is a JWT having 3 parts
        list($header_part, $payload_part, $signature_part) = explode(".", $connect_token);
        $connect_payload = json_decode(base64_decode($payload_part), true);

        // the Connect-Token contains specific data and can be used to avoid hard coded URLs
        $api_base = $connect_payload['fn-ext']['apiBase'];
        $fun_level = $connect_payload['fn-ext']['functionalityLevel'];

        // it is important for your application to be aware of our different stages
        $stage = 'development';
        if ($api_base === 'https://connect.365farmnet.com' || $api_base === 'https://pp-connect.365farmnet.com') {
            $stage = 'production';
        }

        // we suggest validating the Connect-Token to make sure it has been created by us
        $public_key = file_get_contents(__DIR__ . '/365FarmNet_Connect-API_public_key_' . $stage . '.pem');

        try {
            $decoded = JWT::decode($connect_token, $public_key, array('RS256'));
        } catch (Exception $e) {
            $error_msg = 'The Connect-Token found in your URL is invalid!';
        }

        if (isset($decoded)) {

            // the azp-field contains the partner id the token is valid for
            $partner_id = $connect_payload['azp'];
	    $user = $connect_payload['ten'];
            $partner_secret = $partners[$partner_id][$stage];

            // creating the Partner-Token based on your credentials
            $header = array(
                'ver' => '0.1',
                'type' => 'partner'
            );
            $payload = array(
                'con' => $connect_token, // this will also be validated by us
                'iss' => $partner_id
            );

            $partner_token = JWT::encode($payload, $partner_secret, 'HS256', null, $header);

            // accessing one of our APIs using your Partner-Token
            $url = $api_base . '/connect/v1/fields';
            $opts = [
                'http' => [
                    'method' => 'GET',
                    'header' => "Authorization: Bearer {$partner_token}\r\n"
                ]
            ];

            try {
                $context = stream_context_create($opts);
                $contents = file_get_contents($url, false, $context);
                $fields = count(json_decode($contents, true));
            } catch (Exception $e) {
                $error_msg = 'Reading the account\'s fields has failed.';
            }
        }
    } else {
        $error_msg = 'There is no Connect-Token in your URL.';
    }
    ?>
    <div class="error <?php echo (isset($error_msg) ? 'show' : 'hide'); ?>">
        <?php echo $error_msg; ?>
    </div>
    <div id="editor"><?php
	  if (file_exists(CODE_DIR.'/'.$user.'.js')) {
             echo(htmlentities(file_get_contents(CODE_DIR.'/'.$user.'.js')));
          } else {
             echo(htmlentities(file_get_contents(dirname(__FILE__).'/example.js')));
          }
        ?></div>
    <button id="save">Save and Run</button>
    <div id="output">
      <div id="panel">
      </div>
      <div id="console">
      </div>
    </div>
    <script>

        <?= "const user='" . $user . "';\n" ?>

	ace.config.set('basePath', 'https://cdnjs.cloudflare.com/ajax/libs/ace/1.4.7/');
        const editor = ace.edit("editor");
	editor.setTheme("ace/theme/github");
	editor.getSession().setMode("ace/mode/javascript");

        const onSave = function(e) {
		const js = editor.getValue();
		$.post('store.php', {user: user, store: $.trim(js)})
                .done(function(success) {
			showConsole('');
			showPanel('');
                        scenario={};
			eval(js);
		}).fail(function(error) {
			alert(JSON.stringify(error));
		});
		editor.focus();
		e.preventDefault();
	};
        $("#save").click(onSave);

	const console = ace.edit("console");
	console.setTheme("ace/theme/github");

	function showConsole(str) {
	  console.setValue(str);
	  console.clearSelection();
	  const row = console.session.getLength();
	  console.navigateTo(row, 0);
          console.gotoLine(row);
	  console.focus();
        }

	function log(str) {
          let s = console.getValue();
          showConsole(s + '\n' + str);
        }

	function showPanel(html) {
	  $('#panel').html(html);
        }

        const GET = $.get;
        const POST = (url, data) => {
          return $.ajax(url, {
            'data': JSON.stringify(data),
            'type': 'POST',
            'processData': false,
            'contentType': 'application/json'
          });
        };
        const PUT = (url, data) => {
          return $.ajax(url, {
            'data': JSON.stringify(data),
            'type': 'PUT',
            'processData': false,
            'contentType': 'application/json'
          });
        };
        const DELETE = (url) => {
          return $.ajax(url, {
            'type': 'DELETE',
            'contentType': 'application/json'
          });
        };

        let scenario = {};
        <?= "const functionalityLevel='" . $fun_level . "';\n" ?>
	<?= "const apiBase='" . $api_base . "';\n" ?>
	$.ajaxSetup({
	  headers: { Authorization: 'Bearer <?= $partner_token ?>' }
	});

  $(function() {
    const js = editor.getValue();
		eval(js);
  });
    </script>
</body>

</html>
